/*=========================================================================

   Program: ParaView
   Module:  pqDualComboBox.cxx

   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
   All rights reserved.

   ParaView is a free software; you can redistribute it and/or modify it
   under the terms of the ParaView license version 1.2.

   See License_v1.2.txt for the full ParaView license.
   A copy of this license can be obtained by contacting
   Kitware Inc.
   28 Corporate Drive
   Clifton Park, NY 12065
   USA

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

========================================================================*/
#include "pqDualComboBox.h"

#include "pqPropertiesPanel.h"

#include "vtkSMEnumerationDomain.h"
#include "vtkSMProperty.h"
#include "vtkSMPropertyHelper.h"

#include <QVBoxLayout>
#include <QComboBox>
#include <QtDebug>

class pqDualComboBox::pqInternals
{
public:
  QList<QComboBox*> ComboBoxes;
  pqInternals(vtkSMProperty* smproperty, pqDualComboBox* self)
    {
    QVBoxLayout* vbox = new QVBoxLayout(self);
    vbox->setSpacing(pqPropertiesPanel::suggestedVerticalSpacing());
    vbox->setMargin(pqPropertiesPanel::suggestedMargin());

    for (unsigned int cc=0, max = vtkSMPropertyHelper(smproperty).GetNumberOfElements(); cc<max; cc++)
      {
      QComboBox *box = new QComboBox(self);
      vbox->addWidget(box);
      this->ComboBoxes.push_back(box);

      // this ensures that the combo-box is filled with the values from
      // enumeration domain.
      this->fillDomainValues(box, vtkSMEnumerationDomain::SafeDownCast(
          smproperty->GetDomain(
            QString("enum_%1").arg(cc).toLatin1().data())));
      }
    }

  void fillDomainValues(QComboBox* box, vtkSMEnumerationDomain* domain)
    {
    for (unsigned int cc=0, max = domain->GetNumberOfEntries(); cc < max; cc++)
      {
      box->addItem(domain->GetEntryText(cc), domain->GetEntryValue(cc));
      }
    }
};

//-----------------------------------------------------------------------------
pqDualComboBox::pqDualComboBox(
  vtkSMProxy* smproxy, vtkSMProperty* smproperty, QWidget* parentObject)
  : Superclass(smproxy, parentObject),
  Internals(new pqDualComboBox::pqInternals(smproperty, this))
{
  for (int cc=0, max=this->Internals->ComboBoxes.size(); cc<max; cc++)
    {
    // sets up a connection between the smproperty and the QComboBox.
    this->addPropertyLink(this->Internals->ComboBoxes[cc],
      "currentText",
      SIGNAL(currentIndexChanged(const QString &)),
      smproxy,
      smproperty,
      cc);
    }
}

//-----------------------------------------------------------------------------
pqDualComboBox::~pqDualComboBox()
{
}
